package com.orientechnologies.orient.server.distributed;

import com.orientechnologies.orient.core.db.*;
import com.orientechnologies.orient.core.db.document.ODatabaseDocumentTx;
import com.orientechnologies.orient.core.db.tool.ODatabaseExport;
import com.orientechnologies.orient.core.db.tool.ODatabaseImport;
import com.orientechnologies.orient.core.index.OIndex;
import com.orientechnologies.orient.core.metadata.schema.OClass;
import com.orientechnologies.orient.core.metadata.schema.OType;
import com.orientechnologies.orient.core.record.OElement;
import com.orientechnologies.orient.core.sql.executor.OResultSet;
import com.orientechnologies.orient.server.OServer;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import java.io.File;
import java.io.IOException;

import static org.junit.Assert.assertTrue;

public class DistributedDatabaseImportIT {

  private OServer server0;
  private OServer server1;
  private OServer server2;

  @Before
  public void before() throws Exception {
    server0 = OServer.startFromClasspathConfig("orientdb-simple-dserver-config-0.xml");
    server1 = OServer.startFromClasspathConfig("orientdb-simple-dserver-config-1.xml");
    server2 = OServer.startFromClasspathConfig("orientdb-simple-dserver-config-2.xml");
  }

  @Test
  public void test() throws IOException {
    OrientDB ctx1 = server0.getContext();
    ctx1.create("test", ODatabaseType.PLOCAL);
    ODatabaseSession session = ctx1.open("test", "admin", "admin");
    session.createClass("testa");
    ODatabaseExport export = new ODatabaseExport((ODatabaseDocumentInternal) session, "target/export.tar.gz", iText -> {
    });
    export.exportDatabase();
    export.close();
    session.close();

    ctx1.create("test1", ODatabaseType.PLOCAL);
    ODatabaseSession session1 = ctx1.open("test1", "admin", "admin");
    ODatabaseImport imp = new ODatabaseImport((ODatabaseDocumentInternal) session1, "target/export.tar.gz", iText -> {
    });
    imp.importDatabase();
    imp.close();
    session1.close();

    OrientDB ctx2 = server1.getContext();
    ODatabaseSession session2 = ctx2.open("test1", "admin", "admin");
    assertTrue(session2.getMetadata().getSchema().existsClass("testa"));
    session2.close();
  }

  @After
  public void after() throws InterruptedException {
    server0.dropDatabase("test");
    server0.dropDatabase("test1");
    server0.shutdown();
    server1.shutdown();
    server2.shutdown();
    File file = new File("target/export.tar.gz");
    if (file.exists())
      file.delete();
    ODatabaseDocumentTx.closeAll();
  }

}